var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Event } from "../../Event";
import { TypedEvent } from '../../TypedEvent';
import { Api } from "../../Api";
import { ExalusConnectionService } from '../ExalusConnectionService';
import { SessionService } from '../Session/SessionService';
import { DataFrame, Method, Status } from "../../DataFrame";
import { LocalStorageService } from "../LocalStorageService";
import { WebApiCacheService } from "../WebApi/WebApiCacheService";
export class ControllerConfigurationService {
    constructor() {
        this._connection = null;
        this._session = null;
        this._localStorage = null;
        this._onEnteredConfigurationEvent = new Event();
        this._onExitedConfigurationEvent = new Event();
        this._onConfigurationTimeCheckedEvent = new TypedEvent();
        this._configurationTime = new Date();
        this._appEnteredConfigurationMode = false;
        this._didConfigurationChange = false;
        this._synchronizationTaskCompletionSource = new Promise((resolve) => {
            Api.Get(SessionService.ServiceName).OnUserLoggedInEvent().Subscribe((user) => __awaiter(this, void 0, void 0, function* () {
                this._didConfigurationChange = yield this.CheckIfConfigurationHasChangedAsync();
                resolve();
            }));
        });
        this._connection = Api.Get(ExalusConnectionService.ServiceName);
        this._session = Api.Get(SessionService.ServiceName);
        this._localStorage = Api.Get(LocalStorageService.ServiceName);
        this._connection.OnDataReceivedEvent().Subscribe(data => {
            if (data.Resource == "/info/configuration/mode/entry") {
                if (!this._appEnteredConfigurationMode)
                    this._onEnteredConfigurationEvent.Invoke();
            }
            else if (data.Resource == "/info/configuration/mode/exit") {
                if (!this._appEnteredConfigurationMode)
                    this._onExitedConfigurationEvent.Invoke();
            }
        });
    }
    GetServiceName() {
        return ControllerConfigurationService.ServiceName;
    }
    DidCofigurationChangeAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield this._synchronizationTaskCompletionSource;
            return Promise.resolve(this._didConfigurationChange);
        });
    }
    GetLastKnownConfigurationChangeTime() {
        var _a;
        const result = (_a = this._localStorage) === null || _a === void 0 ? void 0 : _a.Read(this.GetServiceName(), "last_configuration_time");
        if (result != null)
            return new Date(result);
        else
            return new Date();
    }
    SaveLastConfigurationChangeTime(time) {
        var _a;
        (_a = this._localStorage) === null || _a === void 0 ? void 0 : _a.Save(this.GetServiceName(), "last_configuration_time", time.toString());
    }
    EnterConfigurationModeAsync() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this._appEnteredConfigurationMode = true;
            yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.SendAndWaitForResponseAsync(new EntryConfigurationModeRequest(), 20000, false));
        });
    }
    ExitConfigurationModeAsync() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this._appEnteredConfigurationMode = true;
            yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.SendAndWaitForResponseAsync(new ExitConfigurationModeRequest(), 20000, false));
            Api.Get(WebApiCacheService.ServiceName).ClearCache();
            this._appEnteredConfigurationMode = false;
        });
    }
    OnEnteredConfigurationEvent() {
        return this._onEnteredConfigurationEvent;
    }
    OnExitedConfigurationEvent() {
        return this._onExitedConfigurationEvent;
    }
    OnConfigurationTimeCheckedEvent() {
        return this._onConfigurationTimeCheckedEvent;
    }
    GetLastConfigurationChangeTimeAsync() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.SendAndWaitForResponseAsync(new LastChangeTimeRequest(), 20000, false));
            if (result == null || (result === null || result === void 0 ? void 0 : result.Status) != Status.OK)
                throw new Error("Failed to get last configuration change time");
            this._configurationTime = new Date(result.Data);
            return this._configurationTime;
        });
    }
    CheckIfConfigurationHasChangedAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.GetLastConfigurationChangeTimeAsync();
            if (this.GetLastKnownConfigurationChangeTime().toString() != result.toString()) {
                this.SaveLastConfigurationChangeTime(result);
                this._onConfigurationTimeCheckedEvent.Invoke(result);
                return true;
            }
            else
                return false;
        });
    }
}
ControllerConfigurationService.ServiceName = "ControllerConfigurationService";
class EntryConfigurationModeRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/system/configuration/mode/entry";
        this.Method = Method.Put;
    }
}
class ExitConfigurationModeRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/system/configuration/mode/exit";
        this.Method = Method.Put;
    }
}
class LastChangeTimeRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/system/configuration/change/time";
        this.Method = Method.Get;
    }
}
//# sourceMappingURL=ControllerConfigurationService.js.map